
/*
** Copyright 1992
** Patrick Sweeney
*/
#ifndef define_LL2
#define define_LL2

#include <stdio.h>

typedef struct _ll2e {
    struct _ll2e *prev;
    struct _ll2e *next;
    char *data;
}

Ll2e;

typedef struct {
    struct _ll2e *first;
    struct _ll2e *last;
} Ll2;

extern Ll2 *ll2_create();
extern void ll2_destroy(Ll2 * ll2, void (*destroyfunc) (char *data));
extern int ll2_append(Ll2 * ll2, char *data);

#endif				/* define_LL2 */
